"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = require("react");
var rsup_duration_1 = require("rsup-duration");
var Waiter = /** @class */ (function () {
    function Waiter() {
        this._countMap = Object.create(null);
        this._listenersMap = Object.create(null);
        this.wait = this.wait.bind(this);
        this.useWait = this.useWait.bind(this);
    }
    Waiter.prototype.isWaiting = function (name) {
        return name in this._countMap;
    };
    Waiter.prototype._emit = function (name) {
        (this._listenersMap[name] || []).forEach(function (fn) { return fn(); });
    };
    Waiter.prototype._subscribe = function (name, listener) {
        var _this = this;
        if (!(name in this._listenersMap))
            this._listenersMap[name] = [];
        var listeners = this._listenersMap[name];
        listeners.push(listener);
        return function () {
            listeners.splice(listeners.indexOf(listener), 1);
            if (listeners.length === 0)
                delete _this._listenersMap[name];
        };
    };
    Waiter.prototype.wait = function (name, order) {
        var _this = this;
        if (this.isWaiting(name)) {
            this._countMap[name]++;
        }
        else {
            this._countMap[name] = 1;
            this._emit(name);
        }
        var onFinally = function () {
            if (--_this._countMap[name] > 0)
                return;
            delete _this._countMap[name];
            _this._emit(name);
        };
        return Promise.resolve(typeof order === 'function' ? order() : order)
            .then(function (v) { return (onFinally(), v); }, function (err) { return (onFinally(), Promise.reject(err)); });
    };
    // tslint:disable-next-line: cognitive-complexity
    Waiter.prototype.useWait = function (name, _a) {
        var _this = this;
        var _b = _a === void 0 ? {} : _a, _c = _b.delay, delay = _c === void 0 ? 0 : _c, _d = _b.duration, duration = _d === void 0 ? 0 : _d;
        var _e = react_1.useState(this.isWaiting(name)), isWaiting = _e[0], setWaiting = _e[1];
        var memoUnSubscriber = react_1.useMemo(function () {
            var delayer = delay > 0 ? rsup_duration_1.default(delay) : null;
            var persister = duration > 0 ? rsup_duration_1.default(duration) : null;
            var next = null;
            var unmounted = false;
            var unsubscribe = _this._subscribe(name, function () {
                if ((delayer && delayer.isDuring) || (persister && persister.isDuring)) {
                    next = _this.isWaiting(name);
                    return;
                }
                if (_this.isWaiting(name)) {
                    if (delayer) {
                        next = true;
                        delayer.start().then(afterDelay);
                    }
                    else {
                        startWaiting();
                    }
                }
                else {
                    setWaiting(false);
                }
            });
            return function () {
                unmounted = true;
                unsubscribe();
            };
            function startWaiting() {
                setWaiting(true);
                if (persister)
                    persister.start().then(afterDuration);
            }
            function afterDelay() {
                if (unmounted)
                    return;
                if (next === true)
                    startWaiting();
                next = null;
            }
            function afterDuration() {
                if (unmounted)
                    return;
                if (next === false)
                    setWaiting(false);
                next = null;
            }
        }, [delay, duration]);
        react_1.useLayoutEffect(function () { return memoUnSubscriber; }, [memoUnSubscriber]);
        return isWaiting;
    };
    return Waiter;
}());
exports.default = Waiter;
